/*
  TROLL (formerly ResHiggs)
  Example with Pseudo Scalar
*/

#include <iomanip>
#include <vector>
#include <algorithm>
#include <iostream>
#include <fstream>
#include <string>
#include <sstream>
#include <stdio.h>
#include <cmath>
#include <ctime>
#include <cstdlib>
#include <sys/stat.h> 
#include <sys/time.h> 
#include <getopt.h> 

#include "./src/resummationQCD.hh"
#include "./src/resummationSCET.hh"

#include "LHAPDF/LHAPDF.h"


using namespace std;




void print_usage() {
  cout << endl
       << "Usage:" << endl
       << " -p  --pdf-set <set>     specify PDF set" << endl
       << " -s  --PDFmember <mem>   specify PDF member of the set (default 0)" << endl
       << " -m  --mass <mass>       set Pseudo Scalar mass [GeV]" << endl
       << " -E  --Ecm <energy>      set collider energy [GeV]" << endl
       << " -f  --fact <murat>      set muF/mA ratio" << endl
       << " -r  --ren <murat>       set muR/mA ratio" << endl
       << " -t  --topmass <mass>    set top quark mass [GeV]" << endl
       << " -P  --prescription <p>  set resummation prescription: A-soft_2 (0), psi-soft_2 (1, default), N-soft (2) [dQCD resummation]" << endl
       << " -g  --g0exp             exponentiate constant function g0 or g0bar [dQCD resummation]" << endl
       << " -C  --collinear         include collinear contributions [SCET resummation]" << endl
       << " -2  --pi2               perform pi^2 resummation [SCET resummation]" << endl
       << endl;
  exit(0);
  return;
}




double muRrat = 1;
double muFrat = 1;
bool g0exp = false;
int prescr = 1;
bool pi2 = false;
bool collinear = false;

const string sorder[4] = {"LO", "NLO", "NNLO", "NNNLO"};
void read_arguments(int argc, char* argv[], double &mA, double &sqrts, string &set, int &member, double &mtop) {
  const char* const short_options = "ht:p:f:r:m:E:P:gs:C2";
  const struct option long_options[] = { { "help", 0, NULL, 'h' },
					 { "topmass", 1, NULL, 't' },
					 { "pdf-set", 1, NULL, 'p' },
					 { "fact", 1, NULL, 'f' },
					 { "ren", 1, NULL, 'r' },
					 { "mass", 1, NULL, 'm' },
					 { "Ecm", 1, NULL, 'E' },
					 { "prescription", 1, NULL, 'P' },
					 { "g0exp", 0, NULL, 'g' },
					 { "PDFmember", 1, NULL, 's' },
					 { "collinear", 0, NULL, 'C' },
					 { "pi2", 0, NULL, '2' },
					 { NULL, 0, NULL, 0 } };
  int next_option;
  ostringstream sset;
  do {
    next_option = getopt_long (argc, argv, short_options, long_options, NULL);
    switch (next_option) {
    case 'h':
      print_usage();
    case 't':
      mtop = strtod(optarg, NULL);
      break;
    case 'p':
      sset << optarg;
      set = sset.str();
      break;
    case 's':
      member = strtol(optarg, NULL, 10);
      break;
    case 'f':
      muFrat = strtod(optarg, NULL);
      break;
    case 'r':
      muRrat = strtod(optarg, NULL);
      break;
    case 'm':
      mA = strtod(optarg, NULL);
      break;
    case 'E':
      sqrts = strtod(optarg, NULL);
      break;
    case 'P':
      prescr = strtol(optarg, NULL, 10);
      break;
    case 'g':
      g0exp = true;
      break;
    case 'C':
      collinear = true;
      break;
    case '2':
      pi2 = true;
      break;
    case '?':
      print_usage();
    case -1: break;
    default: abort();
    }
  }
  while (next_option != -1);
  //
  // messages
  //
  if(prescr==2 && g0exp==true)
    cout << endl << "\033[1m  WARNING:\033[0m "
	 << "\033[0;31m Using N-soft with g0 exponentiated: this is NOT the standard N-soft resummation prescription!\033[0m"
	 << endl << endl;
  //
  return;
}



string PName[] = { "A-soft_2", "psi-soft_2", "N-soft", "Psi-soft_2" };



int main (int argc, char* argv[]) {



  // default values (modified by runtime arguments)
  double mtop = 172.5;  // pole mass
  double mA = 200;
  double sqrts = 8000;
  //string PDFset = "NNPDF30_nnlo_as_0118.LHgrid";
  string PDFset = "NNPDF30_nnlo_as_0118";
  int PDFmember = 0;


  // read runtime arguments
  read_arguments(argc, argv, mA, sqrts, PDFset, PDFmember, mtop);


  // Start time counter
  struct timeval t0, t1;
  gettimeofday(&t0,NULL);


  // init
  double tau = mA*mA/sqrts/sqrts;
  //
  cout << "Pseudo Scalar mass mA = " << mA << " GeV" << endl
       << "Collider energy:        " << sqrts << " GeV" << endl
       << "tau = mA^2/s = " << tau << endl
       << "top    mass =  " << mtop << "  running in the loop" << endl
       << "Renormalization scale muR = " << muRrat << " * mA" << endl
       << "Factorization   scale muF = " << muFrat << " * mA" << endl
       << "dQCD Resummation: prescription:            " << PName[prescr] << endl
       << "dQCD Resummation: constant exponentiation: " << (g0exp ? "Yes" : "No") << endl
       << "SCET Resummation: collinear improvement:   " << (collinear ? "Yes" : "No") << endl
       << "SCET Resummation: Pi2 resummation:         " << (pi2 ? "Yes" : "No") << endl
       << endl;
  //




  // *** Resummation part *** //
  //
  using namespace resumm;
  dQCDResummation resQ(ggA, LHC, PDFset, PDFmember);
  SCETResummation resS(ggA, LHC, PDFset, PDFmember);
  //
  // dQCD settings
  resQ.SetMtop(mtop);
  resQ.SetLogCounting(primed);
  resQ.SetPrescriptionByName (PName[prescr], g0exp);
  //
  // SCET settings
  resS.SetMtop(mtop);
  resS.SetLogCounting(primed);
  if(collinear) resS.SetAPimprovement(AP1);
  resS.UsePi2Resummation(pi2);


  // compute resummed contribution
  // dQCD
  double dQCDxsNLL    = resQ.xsNLL  (mA, sqrts, muFrat, muRrat,   NLO);
  double dQCDxsNNLL   = resQ.xsNNLL (mA, sqrts, muFrat, muRrat,  NNLO);
  double dQCDxsNNNLL2 = resQ.xsNNNLL(mA, sqrts, muFrat, muRrat,  NNLO);
  double dQCDxsNNNLL3 = resQ.xsNNNLL(mA, sqrts, muFrat, muRrat, NNNLO);
  // SCET
  resS.SetmuH(mA);
  double SCETxsNLL    = resS.xsNLL  (mA, sqrts, muFrat, muRrat,   NLO);
  double SCETxsNNLL   = resS.xsNNLL (mA, sqrts, muFrat, muRrat,  NNLO);
  double SCETxsNNNLL2 = resS.xsNNNLL(mA, sqrts, muFrat, muRrat,  NNLO);
  double SCETxsNNNLL3 = resS.xsNNNLL(mA, sqrts, muFrat, muRrat, NNNLO);


  cout << endl
       << "Use this ouput to construct a resummed cross section as (the same PDFs are meant to be used everywhere)" << endl
       << "  sigma(NjLO+NjLL) = sigma(NjLO) + sigma(LO) * Delta_j K(NjLL)" << endl;
  cout << endl;

  cout << "dQCD:" << endl
       << "Delta_1 K(NLL)  = " << dQCDxsNLL << endl
       << "Delta_2 K(NNLL) = " << dQCDxsNNLL << endl
       << "Delta_2 K(N3LL) = " << dQCDxsNNNLL2 << endl
       << "Delta_3 K(N3LL) = " << dQCDxsNNNLL3 << endl
       << endl;

  cout << "SCET:" << endl
       << "Delta_1 K(NLL)  = " << SCETxsNLL << endl
       << "Delta_2 K(NNLL) = " << SCETxsNNLL << endl
       << "Delta_2 K(N3LL) = " << SCETxsNNNLL2 << endl
       << "Delta_3 K(N3LL) = " << SCETxsNNNLL3 << endl
       << endl;



  // Stop time counter
  gettimeofday(&t1,NULL);
  double t=t1.tv_sec-t0.tv_sec+(t1.tv_usec-t0.tv_usec)*0.000001;
  cout << endl << "Total time: " << t << "s" << endl;


  return 0;

}
